{% extends "base.html.jinja" %}

{% block content %}
  <div class="container-fluid mt-3">
    <div class="row text-center">
      <div class="col">
        <h1 class="display-5 fw-bold text-body-emphasis"></h1>
      </div>
    </div>
    <div class="row mt-3 mx-5">
      <div class="col-md-12 px-5">
          {{ figure }}
      </div>
    </div>
  </div>
{% endblock %}